import inspect
import os
import pytest


class Namer(object):
    ClassName = ''
    MethodName = ''
    Directory = ''

    APPROVED = '.approved'
    RECEIVED = '.received'

    def __init__(self, extension='.txt'):
        self.extension_with_dot = extension
        self.set_for_stack(inspect.stack(1))

    def get_class_name(self):
        return self.ClassName

    def get_method_name(self):
        return self.MethodName

    def get_directory(self):
        return self.Directory

    def get_basename(self):
        class_name = "" if (self.ClassName is None) else (self.ClassName + ".")
        return os.path.join(self.Directory, class_name + self.MethodName)

    def get_received_filename(self, basename=None):
        basename = basename or self.get_basename()
        return basename + self.RECEIVED + self.extension_with_dot

    def get_approved_filename(self, basename=None):
        basename = basename or self.get_basename()
        return basename + self.APPROVED + self.extension_with_dot

    def set_for_stack(self, caller):
        frame = self.get_test_frame(caller)
        stacktrace = caller[frame]
        self.MethodName = stacktrace[3]
        self.ClassName = self.get_class_name_for_frame(stacktrace)
        self.Directory = os.path.dirname(stacktrace[1])

    def get_class_name_for_frame(self, stacktrace):
        if "self" not in stacktrace[0].f_locals:
            return os.path.splitext(os.path.basename(stacktrace[1]))[0]
        else:
            return stacktrace[0].f_locals["self"].__class__.__name__

    def get_test_frame(self, caller):
        frameNumber = 1
        for index, frame in enumerate(caller):
            if self.is_test_method(frame):
                frameNumber = index

        return frameNumber

    def is_test_method(self, frame):
        is_unittest_test = ("self" in frame[0].f_locals
               and "_testMethodName" in frame[0].f_locals["self"].__dict__
               and frame[3] is not "__call__"
               and frame[3] is not "run")

        is_pytest_test = frame[3].startswith("test_")

        return is_unittest_test or is_pytest_test



class StringWriter(object):
    contents = ''

    def __init__(self, contents, extension='.txt'):
        self.contents = contents or ''
        self.extension_with_dot = extension

    def write_received_file(self, received_file):
        with open(received_file, 'w') as f:
            f.write(self.contents)
        return received_file



from ..Reporter import Reporter


class FirstWorkingReporter(Reporter):
    def __init__(self, *reporters):
        self.reporters = reporters

    def report(self, received_path, approved_path):
        for r in self.reporters:
            try:
                success = r.report(received_path, approved_path)
                if success:
                    return True
            except:
                pass

        return False



from approvaltests.Reporter import Reporter


class TestingReporter(Reporter):

    def __init__(self):
        self.called = False

    def report(self, approved_path, received_path):
        self.called = True



from subprocess import call
from approvaltests.Reporter import Reporter


class ReceivedFileLauncherReporter(Reporter):

    @staticmethod
    def get_command(approved_path, received_path):
        return ['cmd', '/C', 'start', received_path, '/B']

    def report(self, approved_path, received_path):
        command_array = self.get_command(approved_path, received_path)
        call(command_array)



import json
from threading import local


from approvaltests.ApprovalException import ApprovalException
from approvaltests.FileApprover import FileApprover
from approvaltests.Namer import Namer
from approvaltests.ReceivedFileLauncherReporter import ReceivedFileLauncherReporter
from approvaltests.StringWriter import StringWriter
from approvaltests.reporters.diff_reporter import DiffReporter

DEFAULT_REPORTER = local()


def set_default_reporter(reporter):
    global DEFAULT_REPORTER
    DEFAULT_REPORTER.v = reporter
    

def get_default_reporter():
    if not hasattr(DEFAULT_REPORTER, "v") or DEFAULT_REPORTER.v is None:
        return DiffReporter()
    return DEFAULT_REPORTER.v


def verify(data, reporter=None):
    if reporter is None:
        reporter = get_default_reporter()
    approver = FileApprover()
    namer = get_default_namer()
    writer = StringWriter(data)

    error = approver.verify(namer, writer, reporter)
    if error is not None:
        raise ApprovalException(error)


def get_default_namer():
    return Namer()


def verify_all(header, alist, formatter=None, reporter=None):
    if formatter is None:
        formatter = PrintList().print_item 
    text = header + '\n\n'
    for i in alist:
        text += formatter(i) + '\n'
    verify(text, reporter)


def verify_as_json(object, reporter=None):
    n_ = to_json(object) + "\n"
    verify(n_, reporter)

def to_json(object):
    return json.dumps(
        object,
        sort_keys=True,
        indent=4,
        separators=(',', ': '),
        default=lambda o: o.__dict__)


class PrintList(object):
    index = 0

    @classmethod
    def print_item(cls, x):
        text = str(cls.index) + ') ' + str(x)
        cls.index += 1
        return text



import filecmp
import os


def exists(path):
    return os.path.isfile(path)


class FileApprover(object):
    def verify(self, namer, writer, reporter):

        base = namer.get_basename()
        approved = namer.get_approved_filename(base)
        received = namer.get_received_filename(base)

        writer.write_received_file(received)
        ok = self.verify_files(approved, received, reporter)
        if not ok:
            return "Approval Mismatch"
        return None

    def verify_files(self, approved_file, received_file, reporter):
        if self.are_files_the_same(approved_file, received_file):
            os.remove(received_file)
            return True

        reporter.report(received_file, approved_file)
        return False

    @staticmethod
    def are_files_the_same(approved_file, received_file):
        if not exists(approved_file) or not exists(received_file):
            return False

        if os.stat(approved_file).st_size != os.stat(received_file).st_size:
            return False
        else:
            return filecmp.cmp(approved_file, received_file)



from ..Reporter import Reporter


class MultiReporter(Reporter):
    def __init__(self, *reporters):
        self.reporters = reporters

    def report(self, received_path, approved_path):
        for r in self.reporters:
            r.report(received_path, approved_path)



import pyperclip

from ..Reporter import Reporter


def get_command_text(received_path, approved_path):
    return 'mv -f {0} {1}'.format(received_path, approved_path)


class ClipboardReporter(Reporter):
    def report(self, received_path, approved_path):
        text = get_command_text(received_path, approved_path)
        print(text)
        pyperclip.copy(text)
        return True


class CommandLineReporter(Reporter):
    def report(self, received_path, approved_path):
        text = get_command_text(received_path, approved_path)
        print('\n\n{}\n\n'.format(text))
        return True



from distutils.core import setup
from setuptools import find_packages

setup(
    name='approvaltests',
    version='0.1.14',
    description='Assertion/verification library to aid testing',
    author='ApprovalTests Contributors',
    author_email='jamesrcounts@outlook.com',
    url='https://github.com/approvals/ApprovalTests.Python',
    packages=find_packages(exclude=['tests*']),
    package_data={'approvaltests':['reporters.json']},
    install_requires=['pyperclip==1.5.27',],
)



from .first_working_reporter import FirstWorkingReporter
from .clipboard_reporter import CommandLineReporter
from ..GenericDiffReporterFactory import GenericDiffReporterFactory


class DiffReporter(FirstWorkingReporter):
    def __init__(self):
        factory = GenericDiffReporterFactory()
        
        reporters = list(factory.get_all_reporters())
        reporters.append(CommandLineReporter())
        super(DiffReporter, self).__init__(*reporters)



import os


class Command(object):

    def __init__(self, cmd):
        self.command = cmd

    @staticmethod
    def executable(cmd):
        return os.path.isfile(cmd) and os.access(cmd, os.X_OK)

    def locate(self):
        path, name = os.path.split(self.command)
        if path and self.executable(self.command):
            return self.command
        else:
            for path in os.environ["PATH"].split(os.pathsep):
                path=path.strip('"')
                exe = os.path.join(path, self.command)
                if self.executable(exe):
                    return exe

        return None


import json
import os
import subprocess

from approvaltests.Command import Command
from approvaltests.Reporter import Reporter


class GenericDiffReporter(Reporter):
    @staticmethod
    def create(diff_tool_path):
        return GenericDiffReporter(['custom', diff_tool_path])

    def __init__(self, config):
        self.name = config[0]
        self.path = config[1]

    def __str__(self):
        return json.dumps(
            {
                'name': self.name,
                'path': self.path
            },
            sort_keys=True,
            indent=4,
            separators=(',', ': ')
        )

    @staticmethod
    def create_empty_file(file_path):
        open(file_path, 'w').close()

    @staticmethod
    def run_command(command_array):
        subprocess.call(command_array)

    def get_command(self, received, approved):
        return [
            self.path,
            received,
            approved
        ]

    def report(self, received_path, approved_path):
        if not self.is_working:
            return False
        if not os.path.isfile(approved_path):
            self.create_empty_file(approved_path)
        command_array = self.get_command(received_path, approved_path)
        self.run_command(command_array)
        return True

    def is_working(self):
        return Command(self.path).locate()



import inspect
import json
import os
from approvaltests.GenericDiffReporter import GenericDiffReporter
from approvaltests.Namer import Namer


class GenericDiffReporterFactory(object):
    reporters = []

    def __init__(self):
        directory = os.path.dirname(inspect.stack(1)[0][1])
        path = os.path.join(directory, 'reporters.json')
        self.load(path)

    def list(self):
        return [r[0] for r in self.reporters]

    def get(self, reporter_name):
        config = next((r for r in self.reporters if r[0] == reporter_name), None)
        if not config:
            return None
        return GenericDiffReporter(config)

    def save(self, file_name):
        with open(file_name, 'w') as f:
            json.dump(
                self.reporters,
                f,
                sort_keys=True,
                indent=2,
                separators=(',', ': ')
            )
        return file_name

    def load(self, file_name):
        with open(file_name, 'r') as f:
            self.reporters = json.load(f)
        return self.reporters

    def get_first_working(self):
        working = (i for i in self.get_all_reporters() if i.is_working())
        return next(working, None)
    
    def get_all_reporters(self):
        instances = (GenericDiffReporter(r) for r in self.reporters)
        return instances

    def remove(self, reporter_name):
        self.reporters = [r for r in self.reporters if r[0] != reporter_name]
